package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special.MageSlayerBonus;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special.DropTransformBonus;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special.AllStatsBonus;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special.LeechBlockBonus;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special.BloodyArrowBonus;
import java.util.function.Function;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.mojang.datafixers.kinds.App;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import net.minecraft.world.level.storage.loot.LootContext;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.entity.projectile.AbstractArrow;
import javax.annotation.Nullable;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.MobType;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attribute;
import java.util.function.BiConsumer;
import net.minecraft.network.chat.Component;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.placebo.util.StepFunction;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.placebo.codec.CodecMap;
import dev.shadowsoffire.placebo.codec.CodecProvider;

public abstract class GemBonus implements CodecProvider<GemBonus>
{
    public static final CodecMap<GemBonus> CODEC;
    public static final Codec<Map<LootRarity, StepFunction>> VALUES_CODEC;
    protected final ResourceLocation id;
    protected final GemClass gemClass;
    
    public GemBonus(final ResourceLocation id, final GemClass gemClass) {
        this.id = id;
        this.gemClass = gemClass;
    }
    
    public abstract GemBonus validate();
    
    public abstract boolean supports(final LootRarity p0);
    
    public abstract int getNumberOfUUIDs();
    
    public abstract Component getSocketBonusTooltip(final ItemStack p0, final LootRarity p1);
    
    public void addModifiers(final ItemStack gem, final LootRarity rarity, final BiConsumer<Attribute, AttributeModifier> map) {
    }
    
    public int getDamageProtection(final ItemStack gem, final LootRarity rarity, final DamageSource source) {
        return 0;
    }
    
    public float getDamageBonus(final ItemStack gem, final LootRarity rarity, final MobType type) {
        return 0.0f;
    }
    
    public void doPostAttack(final ItemStack gem, final LootRarity rarity, final LivingEntity user, @Nullable final Entity target) {
    }
    
    public void doPostHurt(final ItemStack gem, final LootRarity rarity, final LivingEntity user, @Nullable final Entity attacker) {
    }
    
    public void onArrowFired(final ItemStack gem, final LootRarity rarity, final LivingEntity user, final AbstractArrow arrow) {
    }
    
    @Nullable
    public InteractionResult onItemUse(final ItemStack gem, final LootRarity rarity, final UseOnContext ctx) {
        return null;
    }
    
    public void onArrowImpact(final ItemStack gemStack, final LootRarity rarity, final AbstractArrow arrow, final HitResult res) {
    }
    
    public float onShieldBlock(final ItemStack gem, final LootRarity rarity, final LivingEntity entity, final DamageSource source, final float amount) {
        return amount;
    }
    
    public void onBlockBreak(final ItemStack gem, final LootRarity rarity, final Player player, final LevelAccessor world, final BlockPos pos, final BlockState state) {
    }
    
    public float getDurabilityBonusPercentage(final ItemStack gem, final LootRarity rarity, @Nullable final ServerPlayer user) {
        return 0.0f;
    }
    
    public float onHurt(final ItemStack gem, final LootRarity rarity, final DamageSource src, final LivingEntity user, final float amount) {
        return amount;
    }
    
    public void getEnchantmentLevels(final ItemStack gem, final LootRarity rarity, final Map<Enchantment, Integer> enchantments) {
    }
    
    public void modifyLoot(final ItemStack gem, final LootRarity rarity, final ObjectArrayList<ItemStack> loot, final LootContext ctx) {
    }
    
    public ResourceLocation getId() {
        return this.id;
    }
    
    public GemClass getGemClass() {
        return this.gemClass;
    }
    
    protected final ResourceLocation getCooldownId(final ItemStack gemStack) {
        final ResourceLocation gemId = GemItem.getGem(gemStack).getId();
        return new ResourceLocation(gemId.m_135827_(), gemId.m_135815_() + "/" + this.getId().m_214298_());
    }
    
    protected static <T extends GemBonus> App<RecordCodecBuilder.Mu<T>, GemClass> gemClass() {
        return (App<RecordCodecBuilder.Mu<T>, GemClass>)GemClass.CODEC.fieldOf("gem_class").forGetter((Function)GemBonus::getGemClass);
    }
    
    public static void initCodecs() {
        register("attribute", AttributeBonus.CODEC);
        register("multi_attribute", MultiAttrBonus.CODEC);
        register("durability", DurabilityBonus.CODEC);
        register("damage_reduction", DamageReductionBonus.CODEC);
        register("enchantment", EnchantmentBonus.CODEC);
        register("bloody_arrow", (Codec<? extends GemBonus>)BloodyArrowBonus.CODEC);
        register("leech_block", (Codec<? extends GemBonus>)LeechBlockBonus.CODEC);
        register("all_stats", (Codec<? extends GemBonus>)AllStatsBonus.CODEC);
        register("drop_transform", (Codec<? extends GemBonus>)DropTransformBonus.CODEC);
        register("mageslayer", (Codec<? extends GemBonus>)MageSlayerBonus.CODEC);
        register("mob_effect", PotionBonus.CODEC);
    }
    
    private static void register(final String id, final Codec<? extends GemBonus> codec) {
        GemBonus.CODEC.register(Apotheosis.loc(id), (Codec)codec);
    }
    
    static {
        CODEC = new CodecMap("Gem Bonus");
        VALUES_CODEC = LootRarity.mapCodec(StepFunction.CODEC);
    }
}
